function love.load()
	padding = 50
	res = 500
	love.graphics.setMode(res+2*padding,res+2*padding)
	love.graphics.setBackgroundColor(255,255,255)
	---------------
	-- states
	---------------
	state = {}
	state.menu = require'menu'
	state.game = require'game'
	state.gameover = require'gameover'
	
	currentState = 'menu'
	
	---------------
	-- image related stuff
	---------------
	imageList = love.filesystem.enumerate('images')
	
	rowAmt = 3
	columnAmt = 3
	
	drawShadow = function()
		love.graphics.push()
		love.graphics.translate(padding,padding)
		love.graphics.setColor(255,255,255,100)
		love.graphics.drawq(imagePuzzle,shadow,0,0)
		love.graphics.pop()
	end
	
	drawImages = function()
		love.graphics.push()
		love.graphics.translate(padding,padding)
		for i = 0,rowAmt-1 do
			for j = 0,columnAmt-1 do
				local qT= quadMatrix[i][j]
				love.graphics.setColor(255,255,255,255)
				love.graphics.drawq(qT.image,qT.quad,j*blockWidth,i*blockHeight)
				if currentState == 'game' and qT.image == imagePuzzle then
					love.graphics.rectangle('line',j*blockWidth,i*blockHeight,blockWidth,blockHeight)
					love.graphics.setColor(100,100,100)
					love.graphics.rectangle('fill',j*blockWidth,i*blockHeight,20,12)
					love.graphics.setColor(255,255,255)
					love.graphics.print(qT.row .. qT.column,j*blockWidth,i*blockHeight)
				end
				
			end
		end
		love.graphics.pop()
	end
	
	drawSelectionRectangle = function()
		love.graphics.push()
		love.graphics.translate(padding,padding)
		love.graphics.setColor(255,0,0)
		love.graphics.rectangle('line',
			currentIndex.column*blockWidth,
			currentIndex.row*blockHeight,
			blockWidth,
			blockHeight)
		love.graphics.pop()
	end
	
	-- initial randomize
	randomizePieces = function()
		local up,down,left,right = 'up','down','left','right'
		local move
		local choice
		for i = 1,100 do
			move = {up,down,left,right}
			-- prevent trying to move out of bound as a choice
			if currentIndex.row == 0 then
				table.remove(move,2)
			elseif currentIndex.row == rowAmt-1 then
				table.remove(move,1)
			end
			if currentIndex.column == 0 then
				table.remove(move,4)
			elseif currentIndex.column == columnAmt-1 then
				table.remove(move,3)
			end
			
			choice = move[math.random(1,#move)]
			keyReleaseEvent[choice]()
		end
	end
	
	---------------
	-- button
	---------------
	
	isInBox = function(x,y,ax0,ay0,ax1,ay1)
		return x < ax1 and x > ax0 and y < ay1 and y > ay0
	end
	
	newButton = function(text,x,y,w,h,textColor,buttonColor,callback)
		local font = love.graphics.newFont(12)
		local button = {text=text,x=x,y=y,w=w,h=h,textColor=textColor,buttonColor=buttonColor}
		button.mousereleased = function(self,x,y,key)
			if isInBox(x,y,self.x,self.y,self.x+w,self.y+h) then
				callback(self)
			end
		end
		button.draw = function(self)
			love.graphics.setFont(font)
			local fh = font:getHeight()
			love.graphics.setColor(buttonColor)
			love.graphics.rectangle('fill',self.x,self.y,self.w,self.h)
			love.graphics.setColor(textColor)
			love.graphics.printf(self.text,self.x,self.y+self.h/2-fh/2,self.w,'center')
		end
		return button
	end
	
	ngbCallback = function(self)
		state.game()
	end
	local bw,bh = 67,50
	newGameButton = newButton('New Game',
		love.graphics.getWidth()/2-bw/2,10,
		bw,bh,
		{255,255,255},{100,100,100},
		ngbCallback)
	
	sbCallback = function(self)
		if rowAmt < 5 then
			rowAmt = rowAmt + 1
			columnAmt = columnAmt + 1
		else
			rowAmt = 3
			columnAmt = 3
		end
		self.text = 'size: ' .. rowAmt .. 'x' .. columnAmt
	end
	local bw,bh = 80,50
	sizeButton = newButton('size: ' .. rowAmt .. 'x' .. columnAmt,
		love.graphics.getWidth()/2-bw/2,newGameButton.y+newGameButton.w+10,
		bw,bh,
		{255,255,255},{100,100,100},
		sbCallback)
	
	---------------
	-- keyboard events
	---------------
	local kre = {}
	keyReleaseEvent = kre
	
	local returnBlankQuad = function(row,column)
		return quadMatrix[row][column]
	end
	
	local doSwap = function(newrow,newcolumn)
		local quad = returnBlankQuad(currentIndex.row,currentIndex.column)
		local otherQuad = quadMatrix[newrow][newcolumn]
		quadMatrix[newrow][newcolumn] = quad
		quadMatrix[currentIndex.row][currentIndex.column] = otherQuad
		currentIndex.row = newrow
		currentIndex.column = newcolumn
	end
	
	kre.down = function()
		if currentIndex.row > 0 then
			local newRow = currentIndex.row - 1
			doSwap(newRow,currentIndex.column)
		end
	end
	kre.up = function()
		if currentIndex.row < rowAmt-1 then
			local newRow = currentIndex.row + 1
			doSwap(newRow,currentIndex.column)
		end
	end
	kre.right = function()
		if currentIndex.column > 0 then
			local newColumn = currentIndex.column - 1
			doSwap(currentIndex.row,newColumn)
		end
	end
	kre.left = function()
		if currentIndex.column < columnAmt-1 then
			local newColumn = currentIndex.column + 1
			doSwap(currentIndex.row,newColumn)
		end
	end
	
	---------------
	-- mosue events
	---------------
	
	local mre = {}
	mouseReleaseEvent = mre
	
	mre.l = function(x,y)
		x,y = x-padding,y-padding
		local di,dj,qT
		for i = 0,rowAmt-1 do
			for j = 0,columnAmt-1 do
				di,dj = math.abs(currentIndex.row - i),math.abs(currentIndex.column-j)
				qT= quadMatrix[i][j]
				if isInBox(x,y,
					j*blockWidth,
					i*blockHeight,
					j*blockWidth+blockWidth,
					i*blockHeight+blockHeight) and di+dj<=1 then
					doSwap(i,j)
					break
				end
			end
		end
	end
	
	---------------
	-- misc methods
	---------------
	
	winCheck = function()
		local numCorrect = 0
		for i = 0,rowAmt-1 do
			for j = 0,columnAmt-1 do
				local qT= quadMatrix[i][j]
				if qT.row == i and qT.column == j then
					numCorrect = numCorrect + 1
				else
					break
				end
			end
		end
		if numCorrect == rowAmt*columnAmt then
			state.gameover()
		end
	end
	
	---------------
	-- start state
	---------------
	
	state.menu()
end